<?php



/**
 * Representa uma classe OWL.
 * @author Joel Azevedo, n39083
 * @author Diogo Ribeiro, n41567
 * @version 1.0
 */
class OWLClass
{
    private $uri;            // URI desta classe
    private $label;          // Rtulo desta classe
    private $superClassURIs; // Array com os URI das superclasses
    private $subClassURIs;   // Array com os URI das subclasses
    private $memberURIs;     // Array com os URI dos membros (individuals)
    
    
    
    /**
     * Construtor.                  
     */
    public function OWLClass()
    {
        $this->uri = "";
        $this->label = "";                                     
        $this->superClassURIs = array();
        $this->subClassURIs = array();
        $this->memberURIs = array();
        
        return $this;
    }
    
    
    
    /**
     * Adiciona uma superclasse s existentes.
     * @param string $superClass O URI da superclasse a adicionar.
     */
    public function addSuperClassURI($superClass)
    {
        array_push($this->superClassURIs, $superClass);
    }
    
    
    
    /**
     * Adiciona uma subclasse s existentes.
     * @param string $superClass O URI da subclasse a adicionar.
     */
    public function addSubClassURI($subClass)
    {
        array_push($this->subClassURIs, $subClass);
    }
    
    
    
    /**
     * Adiciona um membro aos existentes.
     * @param string $member O URI do membro a adicionar.
     */
    public function addMemberURI($member)
    {
        array_push($this->memberURIs, $member);
    }
    
    
    
    /**
     * Devolve o URI desta classe.
     * @return string O URI desta classe.
     */
    public function getURI()
    {
        return $this->uri;
    }
    
    
    
    /**
     * Devolve o rtulo desta classe.
     * @return string O rtulo desta classe.
     */
    public function getLabel()
    {
        return $this->label;
    }
    
    
    
    /**
     * Devolve os URIs das superclasses.
     * @return array Um array com os URIs das superclasses.
     */
    public function getSuperClassURIs()
    {
        return $this->superClassURIs;
    }
    
    
    /**
     * Devolve os URIs das subclasses.
     * NOTA: Antes de chamar este mtodo,  necessrio inferir as subclasses
     * atravs do mtodo OWLReader::inferSubclasses();
     * @return array Um array com os URIs das subclasses.
     */
    public function getSubClassURIs()
    {
        return $this->subClassURIs;
    }
    
    
    
    /**
     * Devolve os URIs dos membros.
     * @return array Um array com os URIs dos membros.
     */
    public function getMemberURIs()
    {
        return $this->memberURIs;
    }
    
    
    
    /**
     * Define o URI desta classe.                  
     * @param string $uri O novo URI da classe.
     */
    function setURI($uri)
    {
        $this->uri = $uri;
    }
    
    
    
    /**
     * Define o rtulo desta classe.
     * @param string $label O novo rtulo desta classe.
     */
    public function setLabel($label)
    {
        $this->label = $label;
    }
}
?>