<?php
require_once "OWLTag.php";
require_once "OWLClass.php";



/**
 * Definio da tag "Class".
 * Subclasse de OWLTag
 * @author Joel Azevedo, n39083
 * @author Diogo Ribeiro, n41567
 * @version 1.0
 */
class OWLClassTag extends OWLTag
{
    /**
     * Constructor.
     * @param OWLOntology $ontology O apontador para onde ser guardada a ontologia.
     * @param OWLTag $currentSuperTag O apontador para a tag de nvel superior. Pode ser <code>null</code>.  
     */
    public function OWLClassTag(&$ontology, $currentSuperTag)
    {
        parent::OWLTag($ontology, $currentSuperTag);
        
        return $this;
    }
    
                                    
    
    /**
     * Manipulador de incio de tag.
     * @param resource $parser A referncia do parser XML.
     * @param string $name O nome do elemento.
     * @param array $attribs Array associativo com os elementos do valor.
     */
    function openTag($parser, $name, $attribs)
    {   
        if($this->currentSubTag != null)
            $this->currentSubTag->openTag($parser, $name, $attribs);
        elseif($this->isTagOpen)
        {
            // Verificar se existe uma classe "especializada" para esta nova tag:
            if(array_key_exists($name, $this->tagHandlerClasses))
                $this->currentSubTag = new $this->tagHandlerClasses[$name](&$this->ontology, $this);
            else $this->currentSubTag = new OWLTag(&$this->ontology, $this);
            $this->currentSubTag->openTag($parser, $name, $attribs);
        }
        else
        {
            // DEBUG --- \/
            if(OWLReader::$DEBUG) {
                print "(ClassTag)  ";
                for($obj = $this->currentSuperTag->currentSuperTag;
                    $obj != null; $obj = $obj->currentSuperTag) print "|  ";
                print "[OPEN] NAME=\"" . $name . "\" ATTRIBS=\"";
                foreach($attribs as $atrib) print $atrib . ";";
                print "\"<br/>";
            }
            // DEBUG --- /\
            
            // Definir variveis desta tag:
            $this->currentTagName = $name;
            $this->currentTagAttribs = $attribs;
            $this->isTagOpen = true;
            
            // Verificar o tipo de modelo:
            if(strcmp($this->currentTagName, OWLTag::$OWL_CLASS) == 0)
            {
                // Verificar se esta  uma declarao ou referncia:
                $uri = trim($this->currentTagAttribs[OWLTag::$RDF_ABOUT]);
                if(strlen($uri) > 0)
                {
                    // Declarao. Obter o nome da classe:
                    $classLabel = "";
                    $classURI = "";
                    if(strncmp("#", $uri, 1) == 0)
                    {
                        $classLabel = substr($uri, 1);
                        $classURI = $this->ontology->getNamespace() . $classLabel;
                    }
                    else
                    {
                        // Classe "Thing"
                        $classLabel = substr($uri, stripos($uri, "#") + 1);
                        $classURI = $uri;
                        $this->ontology->setThingClassURI($uri);
                    }
                    
                    // Adicionar classe  topologia:
                    $class = new OWLClass();
                    $class->setURI($classURI);
                    $class->setLabel($classLabel);
                    $this->ontology->addClass($class);
                }
            }
            elseif(strcmp($this->currentTagName, OWLTag::$OWL2_CLASS) == 0)
            {
                // Verificar se este  uma referncia ou declarao:
                if(strcmp($this->currentSuperTag->currentTagName, OWLTag::$OWL2_DECLARATION) == 0)
                {
                    // Declarao. Obter o nome desta classe:
                    $uri = trim($this->currentTagAttribs["URI"]);
                    
                    // Adicionar classe  topologia:
                    $class = new OWLClass();
                    $class->setURI($uri);
                    $class->setLabel(substr($uri, stripos($uri, "#") + 1));
                    $this->ontology->addClass($class);
                }
                elseif(strcmp($this->currentSuperTag->currentTagName, OWLTag::$OWL2_CLASSASSERTION) == 0)
                {
                    // Referncia. Definir a varivel correcta em OWLClassAssertionTag
                    $this->currentSuperTag->classURI = $this->currentTagAttribs["URI"];
                }
                elseif(strcmp($this->currentSuperTag->currentTagName, OWLTag::$OWL2_SUBCLASSOF) == 0)
                {
                    // Referncia. Definir a varivel correcta em OWLSubClassOfTag:
                    if($this->currentSuperTag->subClassURI == null)
                        $this->currentSuperTag->subClassURI = $this->currentTagAttribs["URI"];
                    else  $this->currentSuperTag->superClassURI = $this->currentTagAttribs["URI"];
                }
                else
                {
                    // Referncia de tipo desconhecido...
                }   
            }
            else
            {
                // Tag "class" no reconhecida...
            } 
            
        }
    }
}
?>