<?php



/**
 * Representa um indivduo OWL.
 * @author Joel Azevedo, n39083
 * @author Diogo Ribeiro, n41567
 * @version 1.0
 */
class OWLIndividual
{
    private $uri;          // URI desta classe
    private $label;        // Rtulo desta classe
    private $types;        // Array com os URI das classes-tipo
    private $OPAssertions; // Array 2D com os URI (propriedade de objecto, indivduo) para asseres de propriedades de objectos
    
    
    
    /**
     * Construtor.                  
     */
    public function OWLIndividual()
    {
        $this->uri = "";
        $this->label = "";                                     
        $this->types = array();
        $this->OPAssertions = array();
        
        return $this;
    }
    
    
    
    /**
     * Adiciona uma classe-tipo s existentes.
     * @param string $uri O URI da classe-tipo a adicionar.
     */
    public function addTypeClass($uri)
    {
        array_push($this->types, $uri);
    }
    
    
    
    /**
     * Adiciona uma relao de propriedade de objecto.
     * @param string $objectPropertyURI O URI da propriedade de objecto.
     * @param string $individualURI O URI do indivduo.
     */
    public function addOPAssertion($objectPropertyURI, $individualURI)
    {
        array_push($this->OPAssertions, Array($objectPropertyURI, $individualURI));
    }
    
    
    
    /**
     * Devolve o URI deste indivduo.
     * @return string O URI deste indivduo.
     */
    public function getURI()
    {
        return $this->uri;
    }
    
    
    
    /**
     * Devolve o rtulo deste indivduo.
     * @return string O rtulo deste indivduo.
     */
    public function getLabel()
    {
        return $this->label;
    }

    
    
    /**
     * Devolve os URIs das classes-tipo.
     * @return array Um array com os URIs das subclasses.
     */
    public function getTypeClassURIs()
    {
        return $this->types;
    }
    
    
    
    /**
     * Devolve as asseres de propriedade de objecto.
     * @return array Um array com arrays de dois elementos, em que o elemento
     * no ndice 0  o URI da propriedade de objecto e o ndice 1  o URI do indivduo.
     */
    public function getOPAssertionURIs()
    {
        return $this->OPAssertions;
    }
    
    
    
    /**
     * Define o URI deste indivduo.                  
     * @param string $uri O novo URI deste indivduo.
     */
    function setURI($uri)
    {
        $this->uri = $uri;
    }
    
    
    
    /**
     * Define o rtulo deste indivduo.
     * @param string $label O novo rtulo deste indivduo.
     */
    public function setLabel($label)
    {
        $this->label = $label;
    }
}
?>