<?php
require_once "OWLTag.php";
require_once "OWLIndividual.php";



/**
 * Definio da tag "Individual".
 * Subclasse de OWLTag
 * @author Joel Azevedo, n39083
 * @author Diogo Ribeiro, n41567
 * @version 1.0
 */
class OWLIndividualTag extends OWLTag
{
    /**
     * Constructor.
     * @param OWLOntology $ontology O apontador para onde ser guardada a ontologia.
     * @param OWLTag $currentSuperTag O apontador para a tag de nvel superior. Pode ser <code>null</code>.  
     */
    public function OWLIndividualTag(&$ontology, $currentSuperTag)
    {
        parent::OWLTag($ontology, $currentSuperTag);
        
        return $this;
    }
    
    
    
    /**                          
     * Manipulador de incio de tag.
     * @param resource $parser A referncia do parser XML.
     * @param string $name O nome do elemento.
     * @param array $attribs Array associativo com os elementos do valor.
     */
    function openTag($parser, $name, $attribs)
    {
        if($this->currentSubTag != null)
            $this->currentSubTag->openTag($parser, $name, $attribs);
        elseif($this->isTagOpen)
        {
            // Verificar se existe uma classe "especializada" para esta nova tag:
            if(array_key_exists($name, $this->tagHandlerClasses))
                $this->currentSubTag = new $this->tagHandlerClasses[$name](&$this->ontology, $this);
            else $this->currentSubTag = new OWLTag(&$this->ontology, $this);
            $this->currentSubTag->openTag($parser, $name, $attribs);
        }
        else
        {
            // DEBUG --- \/
            if(OWLReader::$DEBUG) {
                print "(Individual)";
                for($obj = $this->currentSuperTag->currentSuperTag;
                    $obj != null; $obj = $obj->currentSuperTag) print "|  ";
                print "[OPEN] NAME=\"" . $name . "\" ATTRIBS=\"";
                foreach($attribs as $atrib) print $atrib . ";";
                print "\"<br/>";
            }
            // DEBUG --- /\
            
            // Definir variveis desta tag:
            $this->currentTagName = $name;
            $this->currentTagAttribs = $attribs;
            $this->isTagOpen = true;
            
            // Verificar o tipo de modelo:
            if(strcmp(OWLTag::$OWL2_INDIVIDUAL, $name) == 0)
            {
                // Verificar se esta  uma declarao ou referncia
                if(strcmp(OWLTag::$OWL2_DECLARATION, $this->currentSuperTag->currentTagName) == 0)
                {
                    // Criar o objecto individual:
                    $uri = trim($this->currentTagAttribs["URI"]);
                    $individual = new OWLIndividual();
                    $individual->setURI($uri);
                    $individual->setLabel(substr($uri, stripos($uri, "#") + 1));
                    $this->ontology->addIndividual($individual);
                }
                elseif(strcmp($this->currentSuperTag->currentTagName, OWLTag::$OWL2_CLASSASSERTION) == 0)
                {
                    // Referncia. Definir a varivel correcta em OWLClassAssertionTag:
                    $this->currentSuperTag->individualURI = $this->currentTagAttribs["URI"];
                }
                elseif(strcmp($this->currentSuperTag->currentTagName, OWLTag::$OWL2_OBJECTPROPERTYASSERTION) == 0)
                {
                    // Referncia. Definir a varivel correcta em OWLObjectObjectPropertyAssertionTag:
                    if($this->currentSuperTag->individualURI == null)
                        $this->currentSuperTag->individualURI = $this->currentTagAttribs["URI"];
                    else $this->currentSuperTag->targetIndividualURI = $this->currentTagAttribs["URI"];
                }
                else
                {
                    // Referncia de tipo desconhecido...
                }
            }
            else
            {
                // Tag "Individual" no reconhecida...
            }
        }
    }
}
?>