<?php



/**
 * Representa uma propriedade de objecto OWL.
 * @author Joel Azevedo, n39083
 * @author Diogo Ribeiro, n41567
 * @version 1.0
 */
class OWLObjectProperty
{
    private $uri;               // URI desta classe
    private $label;             // Rtulo desta classe
    private $superPropertyURIs; // Array com os URI das superpropriedades
    private $subPropertyURIs;   // Array com os URI das subpropriedades
    
    
    
    /**
     * Construtor.                  
     */
    public function OWLObjectProperty()
    {
        $this->uri = "";
        $this->label = "";                                     
        $this->superPropertyURIs = array();
        $this->subPropertyURIs = array();
        
        return $this;
    }
    
    
    
    /**
     * Adiciona uma superpropriedade s existentes.
     * @param string $superProperty O URI da superpropriedade a adicionar.
     */
    public function addSuperPropertyURI($superProperty)
    {
        array_push($this->superPropertyURIs, $superProperty);
    }
    
    
    
    /**
     * Adiciona uma subpropriedade s existentes.
     * @param string $subProperty O URI da subpropriedade a adicionar.
     */
    public function addSubPropertyURI($subProperty)
    {
        array_push($this->subPropertyURIs, $subProperty);
    }
    
    
    
    /**
     * Devolve o URI desta propriedade de objecto.
     * @return string O URI desta propriedade de objecto.
     */
    public function getURI()
    {
        return $this->uri;
    }
    
    
    
    /**
     * Devolve o rtulo desta propriedade de objecto.
     * @return string O rtulo desta propriedade de objecto.
     */
    public function getLabel()
    {
        return $this->label;
    }
    
       
    
    /**
     * Devolve os URIs das superpropriedades.
     * @return array Um array com os URIs das superpropriedades.
     */
    public function getSuperPropertyURIs()
    {
        return $this->superPropertyURIs;
    }
    
    
    
    /**
     * Devolve os URIs das subpropriedades.
     * @return array Um array com os URIs das subpropriedades.
     */
    public function getSubPropertyURIs()
    {
        return $this->subPropertyURIs;
    }
    
    
    
    /**
     * Define o URI desta propriedade de objecto.                  
     * @param string $uri O novo URI da propriedade de objecto.
     */
    function setURI($uri)
    {
        $this->uri = $uri;
    }
    
    
    
    /**
     * Define o rtulo desta propriedade de objecto.
     * @param string $label O novo rtulo desta propriedade de objecto.
     */
    public function setLabel($label)
    {
        $this->label = $label;
    }
}
?>