<?php
require_once "OWLClass.php";
require_once "OWLIndividual.php";
require_once "OWLObjectProperty.php";

                         

/**
 * Representa uma ontologia OWL.
 * @author Joel Azevedo, n39083
 * @author Diogo Ribeiro, n41567
 * @version 1.0
 */
class OWLOntology
{
    protected $name;                // Nome interno da ontologia.
    protected $namespace;           // Namespace da ontologia.
    protected $path;                // Caminho completo do ficheiro desta topologia.
    protected $ontologyType;        // Formato da ontologia (ver OWLTag).
    protected $comment;             // Comentrio interno da ontologia.
    
    protected $thingClassURI;       // O objecto PHP da classe "Thing".
    protected $classesLT;           // Hash table para lookup dos objectos PHP representantes das classes.
    protected $individualsLT;       // Hash table para lookup dos objectos PHP representantes dos indivduos.
    protected $objectPropertiesLT;  // Hash table para lookup dos objectos PHP representantes dos indivduos.
    
    public $_superClasses;          // Array 2D utilizado para armazenar temporriamente as relaes classes<->subclasses
    public $_superObjectProperties; // Array 2D utilizado para armazenar temporriamente as relaes propriedades de objecto<->subpropriedades de objecto
    public $_members;               // Array 2D utilizado para armazenar temporriamente as relaes classes<->indivduos
    public $_OPAssertions;          // Array 2D utilizado para armazenar temporriamente as asseres de propriedades de obj. nos indivduos
    
    
    
    /**
     * Construtor.                              
     */
    public function OWLOntology()
    {
        $this->name = "";
        $this->namespace = "";
        $this->path = "";
        $this->ontologyType = "";
        $this->comment = "";
        
        $this->thingClassURI = "";
        $this->classesLT = array();
        $this->individualsLT = array();
        $this->objectPropertiesLT = array();
        
        $this->_superClasses = array();
        $this->_members = array();
        $this->_superObjectProperties = array();
        $this->_OPAssertions = array();
        
        return $this;  
    }
    
    
    
    /**
     * Adiciona uma classe  ontologia.
     * @param OWLClass $class A classe a adicionar
     */
    public function addClass($class)
    {
        $this->classesLT[$class->getURI()] = $class;
    }
    
    
    
    /**
     * Adiciona um indivduo  ontologia.
     * @param OWLIndividual $individual O indivduo a adicionar
     */
    public function addIndividual($individual)
    {
        $this->individualsLT[$individual->getURI()] = $individual;
    }
    
    
    
    /**
    * Adiciona uma propriedade de objecto  ontologia.
    * @param OWLObjectProperty $objectProperty
    */
    public function addObjectProperty($objectProperty)
    {
        $this->objectPropertiesLT[$objectProperty->getURI()] = $objectProperty;
    }
    
    
    
    /**
     * Devolve o nome interno desta ontologia.
     * @return string O nome interno desta ontologia.
     */
    public function getName()
    {
        return $this->name;
    }
    
    
    
    /**
     * Devolve o namespace desta ontologia.
     * @return string O namespace desta ontologia.
     */
    public function getNamespace()
    {
        return $this->namespace;
    }
    
    
    
    /**
    * Devolve a path original da ontologia.
    * @return string A path orginal da ontologia.
    */
    public function getPath()
    {
        return $this->path;
    }
    
    
    
    /**
     * Devolve o tipo de ontologia (URL do XML).
     * @return string O tipo de ontologia.
     */
    public function getOntologyType()
    {
        return $this->ontologyType;
    }
    
    
    
    /**
     * Devolve o comentrio interno da ontologia.
     * @return string O comentrio da ontologia.
     */
    public function getComment()
    {
        return $this->comment;
    }
    
    
    
    /**
     * Devolve o URI da classe "Thing" da ontologia.
     * @return OWLClass O URI da classe "Thing"" da ontologia.
     */
    public function getThingClassURI()
    {
        return $this->thingClassURI;
    }
    
    
    
    /**
     * Devolve os URI das classe desta ontologia.
     * @return array O array com os URIs das classes desta ontologia.
     */
    public function getClassURIs()
    {
        $tmp = array_keys($this->classesLT);
        sort($tmp, SORT_STRING);
        return $tmp;
    }
    
    
    
    /**
     * Devolve os URI dos indivduos desta ontologia.
     * @return array O array com os URIs dos indivduos desta ontologia.
     */
    public function getIndividualURIs()
    {
        $tmp = array_keys($this->individualsLT);
        sort($tmp, SORT_STRING);
        return $tmp;
    }
    
    
    
    /**
     * Devolve os URI das propriedads de objecto desta ontologia.
     * @return array O array com os URIs das propriedades de objecto desta ontologia.
     */
    public function getObjectPropertyURIs()
    {
        $tmp = array_keys($this->objectPropertiesLT);
        sort($tmp, SORT_STRING);
        return $tmp;
    }
    
    
    
    /**
    * Devolve o objecto PHP representante da classe.
    * @param string $uri O URI da classe.
    * @return OWLClass O objecto <code>OWLClass</code> correspondente ao URI
    * indicado, ou <code>null</code> caso este no exista.
    */
    public function getClass($uri)
    {
        return $this->classesLT[$uri];
    }
    
    
    
    /**
    * Devolve o objecto PHP representante do indivduo.
    * @param string $uri O URI do indivduo.
    * @return OWLIndividual O objecto <code>OWLIndividual</code>
    * correspondente ao URI indicado, ou <code>null</code> caso este no
    * exista.
    */
    public function getIndividual($uri)
    {
        return $this->individualsLT[$uri];
    }
    
    
    
    /**
    * Devolve o objecto PHP representante da propriedade de objecto.
    * @param string $uri O URI da propriedade de objecto.
    * @return OWLObjectProperty O objecto <code>OWLObjectProperty</code>
    * correspondente ao URI indicado, ou <code>null</code> caso este no
    * exista.
    */
    public function getObjectProperty($uri)
    {
        return $this->objectPropertiesLT[$uri];
    }
    
    
    
    /**
     * Define o nome interno da ontologia.
     * @param string $name O novo nome interno desta ontologia.
     */
    public function setName($name)
    {
        $this->name = $name;
    }
    
    
    
    /**
     * Define a namespace da topologia.
     * @param string $namespace O novo namespace da ontologia.
     */
    public function setNamespace($namespace)
    {
        $this->namespace = $namespace;
    }
    
    
    
    /**
    * Define a path do ficheiro desta ontologia.
    * @param string $path A path do ficheiro da ontologia.
    */
    public function setPath($path)
    {
        $this->path = $path;   
    }
    
    
    
    /**
     * Define o tipo de ontologia.
     * @param string $ontologyType O novo tipo de ontologia.
     */
    public function setOntologyType($ontologyType)
    {
        $this->ontologyType = $ontologyType;
    }
    
    
    
    /**
     * Define o comentrio.
     * @param string $comment O novo comentrio.
     */
    public function setComment($comment)
    {
        $this->comment = $comment;
    }
    
    
    
    /**
     * Define o URI da classe "Thing" da topologia.
     * @param string $thingClassURI O URI da classe "Thing" da topologia.
     */
    public function setThingClassURI($thingClassURI)
    {
        $this->thingClassURI = $thingClassURI;
    }        
}
?>