<?php
require_once "OWLTag.php";
require_once "OWLClass.php";



/**
 * Definio da tag "Ontology".
 * Subclasse de OWLTag
 * @author Joel Azevedo, n39083
 * @author Diogo Ribeiro, n41567
 * @version 1.0
 */
class OWLOntologyTag extends OWLTag
{
    /**
     * Constructor.
     * @param OWLOntology $ontology O apontador para onde ser guardada a ontologia.
     * @param OWLTag $currentSuperTag O apontador para a tag de nvel superior. Pode ser <code>null</code>.  
     */
    public function OWLOntologyTag(&$ontology, $currentSuperTag)
    {
        parent::OWLTag($ontology, $currentSuperTag);
        
        return $this;
    }
    
                                    
    
    /**
     * Manipulador de incio de tag.
     * @param resource $parser A referncia do parser XML.
     * @param string $name O nome do elemento.
     * @param array $attribs Array associativo com os elementos do valor.
     */
    function openTag($parser, $name, $attribs)
    {    //    NOTA: PAssar  frente se a namespace estiver definida!
        if($this->currentSubTag != null)
            $this->currentSubTag->openTag($parser, $name, $attribs);
        elseif($this->isTagOpen)
        {
            // Verificar se existe uma classe "especializada" para esta nova tag:
            if(array_key_exists($name, $this->tagHandlerClasses))
                $this->currentSubTag = new $this->tagHandlerClasses[$name](&$this->ontology, $this);
            else $this->currentSubTag = new OWLTag(&$this->ontology, $this);
            $this->currentSubTag->openTag($parser, $name, $attribs);
        }
        else
        {
            // DEBUG --- \/
            if(OWLReader::$DEBUG) {
                print "(Ontolgy)   ";
                for($obj = $this->currentSuperTag->currentSuperTag;
                    $obj != null; $obj = $obj->currentSuperTag) print "|  ";
                print "[OPEN] NAME=\"" . $name . "\" ATTRIBS=\"";
                foreach($attribs as $atrib) print $atrib . ";";
                print "\"<br/>";
            }
            // DEBUG --- /\
            
            // Definir variveis desta tag:
            $this->currentTagName = $name;
            $this->currentTagAttribs = $attribs;
            $this->isTagOpen = true;
            
            // Verificar o tipo de modelo:
            if(strcmp($this->currentTagName, OWLTag::$OWL2_ONTOLOGY) == 0)
            {
                // Verificar se j foi definida a namespace e nome da ontologia:
                if(strlen($this->ontology->getNamespace()) == 0)
                {
                    // Definir o namespace desta ontologia:
                    $namespace = $this->currentTagAttribs["URI"] . "#";
                    $this->ontology->setNamespace($namespace);
                    
                    // Definir o nome da ontologia:
                    $rootTag = $this->currentSuperTag;
                    while(strcmp($this->rootTag->currentTagName, "") != 0)
                        $rootTag = $rootTag->currentSuperTag;
                    $this->ontology->setName($rootTag->currentNS[$namespace]);
                    
                    // Definir o tipo de ontologia:
                    $this->ontology->setOntologyType(OWLTag::$OWL2);
                }
            }
            else
            {
                // Tag "ontology" no reconhecida...
            }
        }
    }
    
    
    
    /**
     * Manipulador de fim de tag.
     * @param resource $parser A referncia do parser XML.
     * @param string $name O nome do elemento.                            
     */
    public function closeTag($parser, $name)
    {
        if($this->currentSubTag != null)
        {
            $this->currentSubTag->closeTag($parser, $name);
            if($this->currentSubTag->isOpen() == false)
                $this->currentSubTag = null;
        }
        else
        {
            // Definir variveis desta tag:
            $this->isTagOpen = false;
            
            // DEBUG --- \/
            if(OWLReader::$DEBUG) {
                print "(Ontology)  ";
                for($obj = $this->currentSuperTag->currentSuperTag; $obj != null;
                    $obj = $obj->currentSuperTag) print "|  ";
                print "[CLOSE] NAME=\"" . $name . "<br/>";
            }
            // DEBUG --- /\
            
            if(strcmp(OWLTag::$OWL2_ONTOLOGY, $name) == 0)
            {
                // Adicionar a classe "Thing", caso no exista:
                $thingClassURI = OWLTag::$OWL2 . "#Thing";
                $thingClass = $this->ontology->getClass($thingClassURI);
                if($thingClass == null)
                {
                    $thingClass = new OWLClass();
                    $thingClass->setLabel("Thing");
                    $thingClass->setURI($thingClassURI);
                    $this->ontology->addClass($thingClass);
                    $this->ontology->setThingClassURI($thingClass->getURI());
                }
            }
            else
            {
                // Tag "Ontology" no reconhecida...
            }
        }
    }
}
?>