<?php
require_once "OWLTag.php";
require_once "OWLClass.php";



/**
 * Definio da tag "SubClassOf".
 * Subclasse de OWLTag
 * @author Joel Azevedo, n39083
 * @author Diogo Ribeiro, n41567
 * @version 1.0
 */
class OWLSubClassOfTag extends OWLTag
{
    public $superClassURI; // Utilizado para determinar subclasses no formato OWL2
    public $subClassURI;   // Utilizado para determinar subclasses no formato OWL2
    
    
    
    /**
     * Constructor.
     * @param OWLOntology $ontology O apontador para onde ser guardada a ontologia.
     * @param OWLTag $currentSuperTag O apontador para a tag de nvel superior. Pode ser <code>null</code>.  
     */
    public function OWLSubClassOfTag(&$ontology, $currentSuperTag)
    {
        parent::OWLTag($ontology, $currentSuperTag);
        
        $this->superClassURI = null;
        $this->subClassURI = null;
        
        return $this;
    }
    
    
    
    /**                          
     * Manipulador de incio de tag.
     * @param resource $parser A referncia do parser XML.
     * @param string $name O nome do elemento.
     * @param array $attribs Array associativo com os elementos do valor.
     */
    function openTag($parser, $name, $attribs)
    {
        if($this->currentSubTag != null)
            $this->currentSubTag->openTag($parser, $name, $attribs);
        elseif($this->isTagOpen)
        {
            // Verificar se existe uma classe "especializada" para esta nova tag:
            if(array_key_exists($name, $this->tagHandlerClasses))                                                                                                               
                $this->currentSubTag = new $this->tagHandlerClasses[$name](&$this->ontology, $this);
            else $this->currentSubTag = new OWLTag(&$this->ontology, $this);
            $this->currentSubTag->openTag($parser, $name, $attribs);
        }
        else
        {
            // DEBUG --- \/
            if(OWLReader::$DEBUG) {
                print "(SubClassOf)";
                for($obj = $this->currentSuperTag->currentSuperTag;
                    $obj != null; $obj = $obj->currentSuperTag) print "|  ";
                print "[OPEN] NAME=\"" . $name . "\" ATTRIBS=\"";
                foreach($attribs as $atrib) print $atrib . ";";
                print "\"<br/>";
            }
            // DEBUG --- /\
            
            // Definir variveis desta tag:
            $this->currentTagName = $name;
            $this->currentTagAttribs = $attribs;
            $this->isTagOpen = true;
            
            // Verificar o tipo de modelo:
            if(strcmp(OWLTag::$RDFS_SUBCLASSOF, $name) == 0)
            {
                // Verificar se esta  uma declarao de subclasse vlida:
                if($attribs[OWLTag::$RDF_RESOURCE] != null)
                {
                    // Obter o nome das classes:
                    $namespace = $this->ontology->getNamespace();
                    $superClassURI = "";
                    $subClassURI = $namespace . substr($this->currentSuperTag->currentTagAttribs[OWLTag::$RDF_ABOUT], 1);
                    if(strncmp("#", $attribs[OWLTag::$RDF_RESOURCE], 1) == 0) // Thing?
                        $superClassURI = $namespace . substr($attribs[OWLTag::$RDF_RESOURCE], 1);
                    else $superClassURI = $attribs[OWLTag::$RDF_RESOURCE];
                    
                    // Armazenar esta relao na topologia para mais tarde inferir, se necessrio.
                    if(!is_array($this->ontology->_superClasses[$subClassURI]))
                        $this->ontology->_superClasses[$subClassURI] = array();
                    array_push($this->ontology->_superClasses[$subClassURI], $superClassURI);
                }
            }
            else
            {
                // Tag "SubClassOf" no reconhecida...
            }
        }
    }
    
    
    
    /**
     * Manipulador de fim de tag.
     * @param resource $parser A referncia do parser XML.
     * @param string $name O nome do elemento.                            
     */
    public function closeTag($parser, $name)
    {
        if($this->currentSubTag != null)
        {
            $this->currentSubTag->closeTag($parser, $name);
            if($this->currentSubTag->isOpen() == false)
                $this->currentSubTag = null;
        }
        else
        {
            // Definir variveis desta tag:
            $this->isTagOpen = false;
            
            // DEBUG --- \/
            if(OWLReader::$DEBUG) {
                print "(SubClassOf)";
                for($obj = $this->currentSuperTag->currentSuperTag; $obj != null;
                    $obj = $obj->currentSuperTag) print "|  ";
                print "[CLOSE] NAME=\"" . $name . "<br/>";
            }
            // DEBUG --- /\
            
            // Caso esteja no formato OWL2, necessitamos de retirar o nome das classes.
            if(strcmp(OWLTag::$OWL2_SUBCLASSOF, $this->currentTagName) == 0 &&
               $this->subClassURI != null &&
               $this->superClassURI != null)
            {      
                // Se as URI da classe e subclasse estiverem definidas, adicionar  ontologia:
                if(!is_array($this->ontology->_superClasses[$this->subClassURI]))
                    $this->ontology->_superClasses[$this->subClassURI] = array();
                array_push($this->ontology->_superClasses[$this->subClassURI], $this->superClassURI);
            }
            else
            {
                // Contexto da tag desconhecido...
            }
        }
    }
}
?>