<?php
require_once "OWLTag.php";
require_once "OWLClass.php";



/**
 * Definio da tag "SubObjectPropertyOf".
 * Subclasse de OWLTag
 * @author Joel Azevedo, n39083
 * @author Diogo Ribeiro, n41567
 * @version 1.0
 */
class OWLSubObjectPropertyOfTag extends OWLTag
{
    public $superObjectPropertyURI; // Utilizado para determinar subclasses no formato OWL2
    public $subObjectPropertyURI;   // Utilizado para determinar subclasses no formato OWL2
    
    
    
    /**
     * Constructor.
     * @param OWLOntology $ontology O apontador para onde ser guardada a ontologia.
     * @param OWLTag $currentSuperTag O apontador para a tag de nvel superior. Pode ser <code>null</code>.  
     */
    public function OWLSubObjectPropertyOfTag(&$ontology, $currentSuperTag)
    {
        parent::OWLTag($ontology, $currentSuperTag);
        
        $this->superObjectPropertyURI = null;
        $this->subObjectPropertyURI = null;
        
        return $this;
    }
    
    
    
    /**
     * Manipulador de incio de tag.
     * @param resource $parser A referncia do parser XML.
     * @param string $name O nome do elemento.
     * @param array $attribs Array associativo com os elementos do valor.
     */
    function openTag($parser, $name, $attribs)
    {    
        if($this->currentSubTag != null)
            $this->currentSubTag->openTag($parser, $name, $attribs);
        elseif($this->isTagOpen)
        {   
            // Verificar se existe uma classe "especializada" para esta nova tag:
            if(array_key_exists($name, $this->tagHandlerClasses))                                                                                                               
                $this->currentSubTag = new $this->tagHandlerClasses[$name](&$this->ontology, $this);
            else $this->currentSubTag = new OWLTag(&$this->ontology, $this);
            $this->currentSubTag->openTag($parser, $name, $attribs);
            
            
        }
        else
        {
            // DEBUG --- \/
            if(OWLReader::$DEBUG) {
                print "(SObjPropOf)";
                for($obj = $this->currentSuperTag->currentSuperTag;
                    $obj != null; $obj = $obj->currentSuperTag) print "|  ";
                print "[OPEN] NAME=\"" . $name . "\" ATTRIBS=\"";
                foreach($attribs as $atrib) print $atrib . ";";
                print "\"<br/>";
            }
            // DEBUG --- /\
            
            // Definir variveis desta tag:
            $this->currentTagName = $name;
            $this->currentTagAttribs = $attribs;
            $this->isTagOpen = true;
            
            // Verificar se esta  uma definio de subpropriedade de objecto vlida:
            if(strcmp(OWLTag::$OWL_SUBOBJECTPROPERTYOF, $this->currentTagName) == 0 &&
               strcmp(OWLTag::$OWL_OBJECTPROPERTY, $this->currentSuperTag->currentTagName) == 0 &&
               strlen($this->currentTagAttribs[OWLTag::$RDF_RESOURCE]) > 0 &&
               strlen($this->currentSuperTag->currentTagAttribs[OWLTag::$RDF_ABOUT]) > 0)
            {
                // Adicionar a relao  ontologia para mais tarde inferir, se necessrio:
                $subProperty = $this->ontology->getNamespace() . substr($this->currentSuperTag->currentTagAttribs[OWLTag::$RDF_ABOUT], 1);
                $superProperty = $this->ontology->getNamespace() . substr($this->currentTagAttribs[OWLTag::$RDF_RESOURCE], 1);
                if(!is_array($this->ontology->_superObjectProperties[$subProperty]))
                    $this->ontology->_superObjectProperties[$subProperty] = array();
                array_push($this->ontology->_superObjectProperties[$subProperty], $superProperty);
            }
        }
    }
    
    
    
    /**
     * Manipulador de fim de tag.
     * @param resource $parser A referncia do parser XML.
     * @param string $name O nome do elemento.                            
     */
    public function closeTag($parser, $name)
    {
        if($this->currentSubTag != null)
        {
            $this->currentSubTag->closeTag($parser, $name);
            if($this->currentSubTag->isOpen() == false)
                $this->currentSubTag = null;
        }
        else
        {
            // Definir variveis desta tag:
            $this->isTagOpen = false;
            
            // DEBUG --- \/
            if(OWLReader::$DEBUG) {
                print "(SObjPropOf)";
                for($obj = $this->currentSuperTag->currentSuperTag; $obj != null;
                    $obj = $obj->currentSuperTag) print "|  ";
                print "[CLOSE] NAME=\"" . $name . "<br/>";
            }
            // DEBUG --- /\
            
            // Caso esteja no formato OWL2, necessitamos de retirar o nome das classes.
            if(strcmp(OWLTag::$OWL2_SUBOBJECTPROPERTYOF, $this->currentTagName) == 0 &&
               $this->subObjectPropertyURI != null &&
               $this->superObjectPropertyURI != null)
            {      
                // Se as URI da classe e subclasse estiverem definidas, adicionar  ontologia:
                if(!is_array($this->ontology->_superObjectProperties[$this->subObjectPropertyURI]))
                    $this->ontology->_superObjectProperties[$this->subObjectPropertyURI] = array();
                array_push($this->ontology->_superObjectProperties[$this->subObjectPropertyURI], $this->superObjectPropertyURI);
            }
            else
            {
                // Contexto da tag desconhecido...
            }
        }
    }
}
?>