<?php
require_once "OWLOntology.php";
require_once "OWLReader.php";
require_once "OWLRDFTag.php";
require_once "OWLClassTag.php";                                                 
require_once "OWLOntologyTag.php";
require_once "OWLSubClassOfTag.php";
require_once "OWLIndividualTag.php";
require_once "OWLClassAssertionTag.php";
require_once "OWLObjectPropertyTag.php";
require_once "OWLSubObjectPropertyOfTag.php";
require_once "OWLObjectPropertyAssertionTag.php";
require_once "OWLCommentTag.php";
require_once "OWLConstantTag.php";



/**
 * Definio de tags de ontologias OWL. Utiliza um modelo hierriquico.
 * Utilizado pelo interpretador XML.
 * @author Joel Azevedo, n39083
 * @author Diogo Ribeiro, n41567
 * @version 1.0
 */
class OWLTag
{
    // Formatos suportados:
    protected static $supportedFormats = array
    (
        "owl2xml" => "http://www.w3.org/2006/12/owl2-xml#",
        "xsd"     => "http://www.w3.org/2001/XMLSchema#",
        "rdfs"    => "http://www.w3.org/2000/01/rdf-schema#",
        "rdf"     => "http://www.w3.org/1999/02/22-rdf-syntax-ns#",
        "owl"     => "http://www.w3.org/2002/07/owl#"
    );
    
    // Listagem de tags de atributos:
    public static $XML_BASE                     = "http://www.w3.org/XML/1998/namespace:base";
    public static $XML_LANG                     = "http://www.w3.org/XML/1998/namespace:lang";
    public static $OWL                          = "http://www.w3.org/2002/07/owl";
    public static $OWL_ONTOLOGY                 = "http://www.w3.org/2002/07/owl#:Ontology";
    public static $OWL_CLASS                    = "http://www.w3.org/2002/07/owl#:Class";
    public static $OWL_OBJECTPROPERTY           = "http://www.w3.org/2002/07/owl#:ObjectProperty";
    public static $OWL_SUBOBJECTPROPERTYOF      = "http://www.w3.org/2000/01/rdf-schema#:subPropertyOf";
    public static $OWL2                         = "http://www.w3.org/2006/12/owl2-xml";
    public static $OWL2_ANNOTATION              = "http://www.w3.org/2006/12/owl2-xml#:Annotation";
    public static $OWL2_ONTOLOGY                = "http://www.w3.org/2006/12/owl2-xml#:Ontology";
    public static $OWL2_CLASS                   = "http://www.w3.org/2006/12/owl2-xml#:Class";
    public static $OWL2_CONSTANT                = "http://www.w3.org/2006/12/owl2-xml#:Constant";
    public static $OWL2_DECLARATION             = "http://www.w3.org/2006/12/owl2-xml#:Declaration";
    public static $OWL2_SUBCLASSOF              = "http://www.w3.org/2006/12/owl2-xml#:SubClassOf";
    public static $OWL2_INDIVIDUAL              = "http://www.w3.org/2006/12/owl2-xml#:Individual";
    public static $OWL2_CLASSASSERTION          = "http://www.w3.org/2006/12/owl2-xml#:ClassAssertion";
    public static $OWL2_OBJECTPROPERTY          = "http://www.w3.org/2006/12/owl2-xml#:ObjectProperty";
    public static $OWL2_OBJECTPROPERTYASSERTION = "http://www.w3.org/2006/12/owl2-xml#:ObjectPropertyAssertion";
    public static $OWL2_SUBOBJECTPROPERTYOF     = "http://www.w3.org/2006/12/owl2-xml#:SubObjectPropertyOf";
    public static $RDF_RDF                      = "http://www.w3.org/1999/02/22-rdf-syntax-ns#:RDF";
    public static $RDF_ID                       = "http://www.w3.org/1999/02/22-rdf-syntax-ns#:ID";
    public static $RDF_ABOUT                    = "http://www.w3.org/1999/02/22-rdf-syntax-ns#:about";
    public static $RDF_RESOURCE                 = "http://www.w3.org/1999/02/22-rdf-syntax-ns#:resource";
    public static $RDFS_LABEL                   = "http://www.w3.org/1999/02/22-rdf-syntax-ns#:label";    
    public static $RDFS_SUBCLASSOF              = "http://www.w3.org/2000/01/rdf-schema#:subClassOf";
    public static $RDFS_DOMAIN                  = "http://www.w3.org/2000/01/rdf-schema#:domain";
    public static $RDFS_RANGE                   = "http://www.w3.org/2000/01/rdf-schema#:range";
    public static $RDFS_COMMENT                 = "http://www.w3.org/2000/01/rdf-schema#:comment";

    // Tags com classes prprias para interpretao:
    protected $tagHandlerClasses = array
    (
        "http://www.w3.org/1999/02/22-rdf-syntax-ns#:RDF"             => "OWLRDFTag",
        "http://www.w3.org/2006/12/owl2-xml#:Ontology"                => "OWLOntologyTag",
        "http://www.w3.org/2002/07/owl#:Ontology"                     => "OWLOntologyTag",
        "http://www.w3.org/2006/12/owl2-xml#:Class"                   => "OWLClassTag",
        "http://www.w3.org/2002/07/owl#:Class"                        => "OWLClassTag",
        "http://www.w3.org/2006/12/owl2-xml#:SubClassOf"              => "OWLSubClassOfTag",
        "http://www.w3.org/2000/01/rdf-schema#:subClassOf"            => "OWLSubClassOfTag",
        "http://www.w3.org/2006/12/owl2-xml#:Individual"              => "OWLIndividualTag",
        "http://www.w3.org/2006/12/owl2-xml#:ClassAssertion"          => "OWLClassAssertionTag",
        "http://www.w3.org/2006/12/owl2-xml#:ObjectProperty"          => "OWLObjectPropertyTag",
        "http://www.w3.org/2002/07/owl#:ObjectProperty"               => "OWLObjectPropertyTag",
        "http://www.w3.org/2006/12/owl2-xml#:SubObjectPropertyOf"     => "OWLSubObjectPropertyOfTag",
        "http://www.w3.org/2000/01/rdf-schema#:subPropertyOf"         => "OWLSubObjectPropertyOfTag",
        "http://www.w3.org/2006/12/owl2-xml#:ObjectPropertyAssertion" => "OWLObjectPropertyAssertionTag",
        "http://www.w3.org/2000/01/rdf-schema#:comment"               => "OWLCommentTag",
        "http://www.w3.org/2006/12/owl2-xml#:Constant"                => "OWLConstantTag"
    );
    
    // Variveis:
    protected $ontology;          // Ontologia onde sero guardados os dados.
    protected $currentSuperTag;   // Instncia da classe da tag de nvel superior a esta.
    protected $currentTagAttribs; // Atributos da tag actual.
    protected $currentTagName;    // Nome da tag actual.
    protected $currentSubTag;     // Instncia da classe de nvel inferior a esta.
    protected $currentNS;         // Array 2D de namespaces actuais.
    protected $isTagOpen;         // Indica se esta tag ainda se encontra aberta (vlida).
    
        
    
    /**
     * Constructor.
     * @param OWLOntology $ontology O apontador para onde ser guardada a ontologia.
     * @param OWLTag $currentSuperTag O apontador para a tag de nvel superior. Pode ser <code>null</code>.  
     */
    public function OWLTag(&$ontology, $currentSuperTag)
    {
        $this->ontology = &$ontology;
        $this->currentSuperTag = $currentSuperTag;
        $this->currentTagAttribs = array();
        $this->currentTagName = "";
        $this->currentSubTag = null;
        $this->currentNS = array();
        $this->isTagOpen = ($currentSuperTag == null);
        
        return $this;
    }
    
    
    
    /**
     * Manipulador de declarao de namespace:
     * @param resource $parser O parser XML.
     * @param string $prefix O prefixo do namespace.
     * @param string $uri O namespace.
     */
    public function nsDeclTag($parser, $prefix, $uri)
    {  
        if($this->currentSubTag != null)
            $this->currentSubTag->nsDeclTag($parser, $prefix, $uri);
        else
        {
            // DEBUG --- \/
            if(OWLReader::$DEBUG) {
                print "(Tag)       ";
                for($obj = $this->currentSuperTag; $obj != null;
                    $obj = $obj->currentSuperTag) print "|  ";
                print "[NS] PRE=\"" . $prefix . "\" URI=\"" . $uri . "\"<br/>";
            }
            // DEBUG --- /\
            
            // Adicionar ao array de namespaces:
            $this->currentNS[$uri] = $prefix;
        }
    }
    
                                    
    
    /**
     * Manipulador de incio de tag.
     * @param resource $parser A referncia do parser XML.
     * @param string $name O nome do elemento.
     * @param array $attribs Array associativo com os elementos do valor.
     */
    function openTag($parser, $name, $attribs)
    {    
        if($this->currentSubTag != null)
            $this->currentSubTag->openTag($parser, $name, $attribs);
        elseif($this->isTagOpen)
        {   
            // Verificar se existe uma classe "especializada" para esta nova tag:
            if(array_key_exists($name, $this->tagHandlerClasses))                                                                                                               
                $this->currentSubTag = new $this->tagHandlerClasses[$name](&$this->ontology, $this);
            else $this->currentSubTag = new OWLTag(&$this->ontology, $this);
            $this->currentSubTag->openTag($parser, $name, $attribs);
        }
        else
        {
            // DEBUG --- \/
            if(OWLReader::$DEBUG) {
                print "(Tag)       ";
                for($obj = $this->currentSuperTag->currentSuperTag;
                    $obj != null; $obj = $obj->currentSuperTag) print "|  ";
                print "[OPEN] NAME=\"" . $name . "\" ATTRIBS=\"";
                foreach($attribs as $atrib) print $atrib . ";";
                print "\"<br/>";
            }
            // DEBUG --- /\
            
            // Definir variveis desta tag:
            $this->currentTagName = $name;
            $this->currentTagAttribs = $attribs;
            $this->isTagOpen = true;
            
            // Excepo  regra - Verificar se esta  uma declarao de indivduo em RDF/XML:
            $a = $this->ontology->getNamespace();
            $b = substr($name, 0, stripos($name, "#") + 1);
            if((strcmp($name, OWLTag::$OWL . "#:Thing") == 0 ||
                strcmp($this->ontology->getNamespace(), substr($name, 0, stripos($name, "#") + 1)) == 0))
            {
                // Verificar se  uma declarao ou referncia:
                if(strlen($this->currentTagAttribs[OWLTag::$RDF_ABOUT]) > 0)
                {
                    // Declarao. Retirar os ':' do nome da classe:
                    $className = ereg_replace("#:", "#", $name);
                                          
                    // Criar e adicionar o indivduo:
                    $uri = $this->ontology->getNamespace() . substr(trim($this->currentTagAttribs[OWLTag::$RDF_ABOUT]), 1);
                    $individual = new OWLIndividual();
                    $individual->setURI($uri);
                    $individual->setLabel(substr(trim($this->currentTagAttribs[OWLTag::$RDF_ABOUT]), 1));
                    $this->ontology->addIndividual($individual);
                    
                    // Adicionar a relao de membro de classe:
                    if(!is_array($this->ontology->_members[$uri]))
                        $this->ontology->_members[$uri] = array();
                    array_push($this->ontology->_members[$uri], $className);
                }
                elseif(strlen($this->currentTagAttribs[OWLTag::$RDF_RESOURCE]) > 0)
                {
                    // Referncia (assero de propriedade de objecto). Obter variveis:
                    $individualURI = $this->ontology->getNamespace() . substr($this->currentSuperTag->currentTagAttribs[OWLTag::$RDF_ABOUT], 1);
                    $objectPropertyURI = ereg_replace("#:", "#", $name);
                    $targetIndividualURI = $this->ontology->getNamespace() . substr($this->currentTagAttribs[OWLTag::$RDF_RESOURCE], 1);
                    
                    // Armazenar esta relao para mais tarde inferir, se necessrio:
                    if(!is_array($this->ontology->_OPAssertions[$individualURI]))
                        $this->ontology->_OPAssertions[$individualURI] = array();
                    array_push($this->ontology->_OPAssertions[$individualURI], $objectPropertyURI, $targetIndividualURI);                    
                }
            }
        }
    }
    
    
    
    /**
     * Manipulador de fim de tag.
     * @param resource $parser A referncia do parser XML.
     * @param string $name O nome do elemento.                            
     */
    public function closeTag($parser, $name)
    {
        if($this->currentSubTag != null)
        {
            $this->currentSubTag->closeTag($parser, $name);
            if($this->currentSubTag->isOpen() == false)
                $this->currentSubTag = null;
        }
        else
        {
            // Definir variveis desta tag:
            $this->isTagOpen = false;
            
            // DEBUG --- \/
            if(OWLReader::$DEBUG) {
                print "(Tag)       ";
                for($obj = $this->currentSuperTag->currentSuperTag; $obj != null;
                    $obj = $obj->currentSuperTag) print "|  ";
                print "[CLOSE] NAME=\"" . $name . "<br/>";
            }
            // DEBUG --- /\
            
            // Processar aqui...
        }
    }
    
    
    
    /**
     * Manipulador de texto no interior das tags.
     * @param resource $parser A referncia do parser XML.
     * @param string $data O texto no interior da tag.
     */
    public function data($parser, $data)
    {
        if($this->currentSubTag != null)
            $this->currentSubTag->data($parser, $data);
        else
        {
			// DEBUG --- \/
            if(OWLReader::$DEBUG && strlen(trim($data)) != 0) {
                print "(Tag)       ";
                for($obj = $this->currentSuperTag; $obj != null;
                    $obj = $obj->currentSuperTag) print "|  ";
                print "[CHAR] DATA=\"" . trim($data) . "\"<br/>";
            }
            // DEBUG --- /\
            
            // Processar aqui...
        }
    }
    
    
    
    /**
     * Indica se esta tag ainda se encontra aberta.
     * @return boolean <code>true</code> caso se encontre aberta;
     * <code>false</code> em caso negativo.
     */
    public function isOpen()
    {
        return $this->isTagOpen;
    }  
}
?>